/**
 * COPYRIGHT 2024 Flirc, Inc. All rights reserved.
 *
 * This copyright notice is Copyright Management Information under 17 USC 1202
 * and is included to protect this work and deter copyright infringement.
 * Removal or alteration of this Copyright Management Information without
 * the express written permission of Flirc, Inc. is prohibited, and any
 * such unauthorized removal or alteration will be a violation of federal law.
 *
 * @file
 * @brief   Useful utilities
 */

#ifndef I__UTIL_H__
#define I__UTIL_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cpluspuls */

/*******************************************************************************
 * LUT
 *  -- external lookup table
 */
extern uint8_t lookup[256];

#define bitrev8(x)      lookup[(x) & 0xFF]

#define bitrev16(x)   (x = bitrev8((x)>>8)<<8 | bitrev8(x))
#define bitrev24(x)   (x = bitrev8((x)>>16)<<16 | bitrev8((x)>>8)<<8 | \
                                bitrev8(x))
#define bitrev32(x)   (x = bitrev8((x)>>24) | bitrev8((x)>>16)<<8 | \
                                bitrev8((x)>>8)<<16 | bitrev8(x)<<24)
#define bitrev48(x)   (x = (\
			    ((uint64_t)bitrev8(x >> 40)       | \
                            ((uint64_t)bitrev8(x >> 32)) <<8  | \
                            ((uint64_t)bitrev8(x >> 24)) <<16 | \
                            ((uint64_t)bitrev8(x >> 16)) <<24 | \
                            ((uint64_t)bitrev8(x >> 8))  <<32 | \
                            ((uint64_t)bitrev8(x))       <<40 )))

/**
 * ssprintf() formats and stores a series of characters and values in the array 'str'.
 * The 'format' parameter determines the format of the output.
 * The function takes a variable number of arguments.
 *
 * @param *str    - Pointer to a buffer where the resulting C-string is stored
 * @param size    - Maximum number of bytes to be used in the buffer
 * @param *format - C-string that contains a format string followed by an optional
 *                  sequence of arguments
 * @param ...     - Depending on the format string, the function may expect a
 *                  sequence of additional arguments
 */
void ssprintf(char *str, size_t size, const char *format, ...);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* I__UTIL_H__ */
